import os
import requests
from selenium.webdriver.edge.service import Service
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from selenium import webdriver


# Function to check network availability
def is_network_available():
    try:
        # Attempt to access the download URL for Edge Chromium driver to check network connection
        response = requests.get("https://msedgedriver.azureedge.net/", timeout=3)
        return response.status_code == 200
    except requests.ConnectionError:
        return False


# Function to get the default download path for Edge Chromium driver
def get_default_driver_path():
    manager = EdgeChromiumDriverManager()
    return manager.install()


# Function to get the driver version from cache
def get_cached_driver_version(manager):
    cache_dir = manager._cache_manager._root_dir  # Get the root path of the cache directory
    os_type = manager._os_system_manager.get_os_type()  # Get the OS type
    driver_name = manager.driver.get_name()

    # List all versions in the cache directory
    versions_dir = os.path.join(cache_dir, "drivers", driver_name, os_type)
    if os.path.exists(versions_dir):
        versions = os.listdir(versions_dir)
        if versions:
            # Return the latest version found in the cache
            return max(versions)
    return None


# Function to check if the driver has already been downloaded
def get_driver_path():
    network_available = is_network_available()
    print("Network status:", "Available" if network_available else "Unavailable")
    manager = EdgeChromiumDriverManager()
    if network_available:
        return get_default_driver_path()
    else:
        # If no network, check the cache directory
        driver_version = get_cached_driver_version(manager)
        if driver_version:
            driver_name = manager.driver.get_name()
            os_type = manager._os_system_manager.get_os_type()
            cache_dir = manager._cache_manager._root_dir

            # Determine the correct driver filename based on the OS
            if os.name == 'nt':  # Windows
                driver_filename = "msedgedriver.exe"
            else:  # macOS and Linux
                driver_filename = "msedgedriver"

            # Construct the driver path
            driver_path = os.path.join(cache_dir, "drivers", driver_name, os_type, driver_version, driver_filename)
            # Check if the driver path exists
            if os.path.exists(driver_path):
                return driver_path
        raise Exception("Network unavailable and no downloaded driver found")


# Function to get the Edge driver service
def get_edge_driver_service():
    driver_path = get_driver_path()
    print("driver_path", driver_path)
    # Check if the driver path exists
    if os.path.exists(driver_path):
        return Service(driver_path)
    else:
        raise Exception("Network unavailable and no downloaded driver found")


from langchain_core.messages import HumanMessage, AIMessage
from .agent import INPUT_MESSAGE, INTERRUPT_MESSAGE, output, CHAT_DATA, clear_chat_data
from .agent_chat import agent_chat
import json
import threading
import queue
import time
import asyncio


class PeekableQueue(queue.Queue):
    def peek(self):
        try:
            return self._get()
        except queue.Empty:
            return None

def chat(bot_setting_file: str):
    history = []
    input_queue = PeekableQueue()

    def input_thread(input_queue: PeekableQueue):
        while True:
            input_text = input()
            if input_text == INTERRUPT_MESSAGE or input_text.startswith(INPUT_MESSAGE):
                input_queue.put(input_text)
            else:
                raise ValueError("Invalid message")
            

    input_thread_instance = threading.Thread(target=input_thread, args=(input_queue,))
    input_thread_instance.start()

    try:
        while True:
            if not input_queue.empty():
                input_text = input_queue.get()
                if input_text == INTERRUPT_MESSAGE:
                    continue
                try:
                    message = json.loads(input_text[len(INPUT_MESSAGE):])
                    history.append(HumanMessage(message["content"]))
                    clear_chat_data()
                    
                    # 为每个线程创建一个独立的退出标志
                    should_exit = threading.Event()
                    agent_chat_thread = threading.Thread(target=asyncio.run, args=(agent_chat(bot_setting_file, history, should_exit),))
                    agent_chat_thread.start()
                    
                    next_input_text = ""
                    while agent_chat_thread.is_alive():
                        if not input_queue.empty():
                            next_input_text = input_queue.peek()
                            if next_input_text == INTERRUPT_MESSAGE:
                                history.append(AIMessage(content=CHAT_DATA["info"]))
                                should_exit.set()  # 设置退出标志
                                break
                        time.sleep(0.1)  # 每隔 0.1 秒检查一次
                    if next_input_text != INTERRUPT_MESSAGE:
                        history.append(AIMessage(content=CHAT_DATA["output"]))
                except json.JSONDecodeError:
                    print("Invalid JSON format in input message.")
                except KeyError:
                    print("Missing 'content' key in input message.")
                except Exception as e:
                    print(f"An error occurred: {e}")
    finally:
        input_thread_instance.join()



def get_chat_response(bot_setting_file: str, input_text: str):
    history = [HumanMessage(input_text)]
    clear_chat_data()
    should_exit = threading.Event()
    asyncio.run(agent_chat(bot_setting_file, history, should_exit))
    return CHAT_DATA["output"]


import sys
import os
import importlib
from langchain.agents import tool as to_agent_tool
import types
import langchain_core
from .lib import ChangeDirectoryAndPath, ModuleManager, wrapped_func
import json
from pycoze import utils

bot_index  = 0

params = utils.read_params_file()

def ref_bot(bot_id, as_agent_tool=False, workspace_path=None):
    global bot_index
    if workspace_path is None:
        workspace_path = params["workspacePath"]
    tool_base_path = os.path.join(workspace_path, "User/Local/bot")
    module_path = os.path.join(tool_base_path, bot_id)
    module_path = os.path.normpath(os.path.abspath(module_path))
    
    if not os.path.exists(module_path):
        print(f"Bot {bot_id} not found in:"+ module_path)
        return None
    
    try:
        with ModuleManager(module_path) as manager:
            info = module_path + "/info.json"
            with open(info, "r", encoding="utf-8") as f:
                info = json.load(f)
            name = info["name"]
            random_name = "bot_" + str(bot_index)
            bot_index += 1
            function_code = f"""
def {random_name}(command:str) -> str:
    \"\"\"接收任意指令字符串，并返回AI角色（{name}专家）深入思考和执行指令后的字符串结果。
    AI角色（{name}专家）擅长使用各种工具，并会给出专业且更为准确的结果。

    Args:
        command (str): AI角色（{name}专家）需要执行的指令字符串。

    Returns:
        str: AI角色（{name}专家）执行指令后的结果。
    \"\"\"
    from pycoze import bot
    from pycoze import utils
    import tempfile
    
    with tempfile.NamedTemporaryFile(delete=True, mode='w+t') as temp_file:
        sys.stdout = temp_file  # 将输出重定向到临时文件，防止影响AI结果
        result = bot.get_chat_response("botSetting.json", command)
        sys.stdout = sys.__stdout__  # 恢复标准输出
    return result
"""
            exec(function_code)
            tool = to_agent_tool(eval(random_name))
            tool.func = wrapped_func(tool, module_path)
            if tool.description is None:
                tool.description = "This tool is used to " + tool.name + "."
            return tool if as_agent_tool else tool.func
    except Exception as e:
        print(f"Error loading bot {bot_id}: {e}")
        return None